@@ -15,13 +15,14 @@
  */
 #include <decoding.h>
 #include <rlc.h>
+#include <rlc_window.h>
 #include <gprs_debug.h>
 #include <egprs_rlc_compression.h>
 
 extern "C" {
 #include <osmocom/core/utils.h>
 #include <osmocom/core/bitcomp.h>
-#include <osmocom/gprs/protocol/gsm_04_60.h>
+#include <osmocom/gsm/protocol/gsm_44_060.h>
 }
 
 #include <arpa/inet.h>
