@@ -11,10 +11,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include <osmocom/gprs/gprs_bssgp.h>
@@ -156,7 +152,7 @@ static int handle_ran_info_response_nacc(const struct bssgp_ran_inf_app_cont_nac
 	}
 	entry = si_cache_add(bts->pcu->si_cache, &nacc->reprt_cell, &val);
 
-	llist_for_each(tmp, bts_ms_list(bts)) {
+	llist_for_each(tmp, &bts->ms_list) {
 		struct GprsMs *ms = llist_entry(tmp, typeof(*ms), list);
 		if (ms->nacc && nacc_fsm_is_waiting_si_resolution(ms->nacc, &nacc->reprt_cell))
 			osmo_fsm_inst_dispatch(ms->nacc->fi, NACC_EV_RX_SI, entry);
