@@ -34,6 +34,7 @@ enum {
 	DRLCMACUL,
 	DRLCMACSCHED,
 	DRLCMACMEAS,
+	DMS,
 	DTBF,
 	DTBFDL,
 	DTBFUL,
@@ -41,6 +42,7 @@ enum {
 	DPCU,
 	DNACC,
 	DRIM,
+	DE1,
 	aDebug_LastEntry
 };
 
