@@ -191,7 +191,7 @@ static struct msgb *create_packet_cell_chg_continue(const struct nacc_fsm_ctx *c
 	uint8_t tfi_is_dl = tbf_direction(tbf) == GPRS_RLCMAC_DL_TBF;
 	uint8_t tfi = tbf_tfi(tbf);
 	uint8_t container_id = 0;
-	write_packet_cell_change_continue(mac_control_block, 1, rrbp, tfi_is_dl, tfi, true,
+	write_packet_cell_change_continue(mac_control_block, 1, rrbp, tfi_is_dl, tfi, ctx->neigh_key_present,
 			ctx->neigh_key.tgt_arfcn, ctx->neigh_key.tgt_bsic, container_id);
 	LOGP(DNACC, LOGL_DEBUG, "+++++++++++++++++++++++++ TX : Packet Cell Change Continue +++++++++++++++++++++++++\n");
 	rc = encode_gsm_rlcmac_downlink(&bv, mac_control_block);
@@ -270,19 +270,94 @@ static int fill_rim_ran_info_req(const struct nacc_fsm_ctx *ctx, struct bssgp_ra
 	return 0;
 }
 
+/* Generate neighbour key information from a given PacketCellCHangeNotification and a BTS object (lac and cid). The
+ * ctx variable is used for logging only. */
 static int fill_neigh_key_from_bts_pkt_cell_chg_not(struct neigh_cache_entry_key *neigh_key,
+						    bool *neigh_key_present,
 						    const struct gprs_rlcmac_bts *bts,
-						    const Packet_Cell_Change_Notification_t *notif)
+						    const Packet_Cell_Change_Notification_t *notif,
+						    const struct nacc_fsm_ctx *ctx)
 {
+	const Target_Cell_GSM_Notif_t *notif_gsm;
+	const Target_Cell_3G_Notif_t *notif_3g;
+	const Target_Cell_4G_Notif_t *notif_4g;
+
+	memset(neigh_key, 0, sizeof(*neigh_key));
+	*neigh_key_present = false;
+
 	switch (notif->Target_Cell.UnionType) {
-	case 0: /* GSM */
+	case 0:	/* GSM */
+		notif_gsm = &notif->Target_Cell.u.Target_Cell_GSM_Notif;
+		LOGPFSML(ctx->fi, LOGL_NOTICE, "TargetCell: RAT=GSM, ARFCN=%u, BSIC=%u\n",
+			 notif_gsm->ARFCN, notif_gsm->BSIC);
+
 		neigh_key->local_lac = bts->cgi_ps.rai.lac.lac;
 		neigh_key->local_ci = bts->cgi_ps.cell_identity;
-		neigh_key->tgt_arfcn = notif->Target_Cell.u.Target_Cell_GSM_Notif.ARFCN;
-		neigh_key->tgt_bsic = notif->Target_Cell.u.Target_Cell_GSM_Notif.BSIC;
+		neigh_key->tgt_arfcn = notif_gsm->ARFCN;
+		neigh_key->tgt_bsic = notif_gsm->BSIC;
+		*neigh_key_present = true;
 		return 0;
 	default:
-		return -ENOTSUP;
+		switch (notif->Target_Cell.u.Target_Other_RAT_Notif.UnionType) {
+		case 0:	/* UTRAN */
+			notif_3g = &notif->Target_Cell.u.Target_Other_RAT_Notif.u.Target_Cell_3G_Notif;
+			if (notif_3g->Exist_FDD_Description) {
+				LOGPFSML(ctx->fi, LOGL_NOTICE,
+					 "TargetCell: RAT=UTRAN, FDD-ARFCN=%u => no system information provided.\n",
+					 notif_3g->FDD_Target_Cell_Notif.FDD_ARFCN);
+			} else if (notif_3g->Exist_TDD_Description) {
+				LOGPFSML(ctx->fi, LOGL_NOTICE,
+					 "TargetCell: RAT=UTRAN, TDD-ARFCN=%u => no system information provided.\n",
+					 notif_3g->TDD_Target_Cell.TDD_ARFCN);
+			}
+			return 0;
+		default:
+			switch (notif->Target_Cell.u.Target_Other_RAT_Notif.u.Target_Other_RAT_2_Notif.UnionType) {
+			case 0:	/* E-UTRAN (and older RAT) */
+				notif_4g =
+				    &notif->Target_Cell.u.Target_Other_RAT_Notif.u.Target_Other_RAT_2_Notif.u.Target_Cell_4G_Notif;
+				notif_3g = &notif_4g->Target_Cell_3G_Notif;
+				if (notif_4g->Exist_Arfcn) {
+					LOGPFSML(ctx->fi, LOGL_NOTICE, "TargetCell: RAT=GSM, ARFCN=%u, BSIC=%u\n",
+						 notif_4g->Arfcn, notif_4g->bsic);
+					neigh_key->local_lac = bts->cgi_ps.rai.lac.lac;
+					neigh_key->local_ci = bts->cgi_ps.cell_identity;
+					neigh_key->tgt_arfcn = notif_4g->Arfcn;
+					neigh_key->tgt_bsic = notif_4g->bsic;
+					*neigh_key_present = true;
+					return 0;
+				}
+				if (notif_4g->Exist_3G_Target_Cell) {
+					if (notif_3g->Exist_FDD_Description) {
+						LOGPFSML(ctx->fi, LOGL_NOTICE,
+							 "TargetCell: RAT=UTRAN, FDD-ARFCN=%u => no system information provided.\n",
+							 notif_3g->FDD_Target_Cell_Notif.FDD_ARFCN);
+					} else if (notif_3g->Exist_TDD_Description) {
+						LOGPFSML(ctx->fi, LOGL_NOTICE,
+							 "TargetCell: RAT=UTRAN, TDD-ARFCN=%u => no system information provided.\n",
+							 notif_3g->TDD_Target_Cell.TDD_ARFCN);
+					}
+					return 0;
+				}
+				if (notif_4g->Exist_Eutran_Target_Cell) {
+					LOGPFSML(ctx->fi, LOGL_NOTICE,
+						 "TargetCell: RAT=E-UTRAN, EARFCN=%u, CI=%u => no system information provided.\n",
+						 notif_4g->Target_EUTRAN_Cell.EARFCN,
+						 notif_4g->Target_EUTRAN_Cell.Physical_Layer_Cell_Identity);
+					return 0;
+				}
+
+				/* TODO: do something meaningful with an Eutran_Ccn_Measurement_Report, in case it is
+				 * provided. */
+
+				LOGPFSML(ctx->fi, LOGL_NOTICE, "TargetCell: (none, invalid)\n");
+				return -EINVAL;
+			default:
+				LOGPFSML(ctx->fi, LOGL_NOTICE,
+					 "TargetCell: RAT=CSG-UTRAN|CSG-EUTRAN, (not supported)\n");
+				return -ENOTSUP;	/* TODO: Add support */
+			}
+		}
 	}
 }
 
@@ -316,17 +391,28 @@ static void handle_retrans_pkt_cell_chg_notif(struct nacc_fsm_ctx *ctx, const Pa
 {
 	struct gprs_rlcmac_bts *bts = ctx->ms->bts;
 	struct neigh_cache_entry_key neigh_key;
+	bool neigh_key_present;
+	int rc;
 
-	if (fill_neigh_key_from_bts_pkt_cell_chg_not(&neigh_key, bts, notif) < 0) {
-		LOGPFSML(ctx->fi, LOGL_NOTICE, "TargetCell type=0x%x not supported\n",
-			 notif->Target_Cell.UnionType);
+	rc = fill_neigh_key_from_bts_pkt_cell_chg_not(&neigh_key, &neigh_key_present, bts, notif, ctx);
+	if (rc < 0) {
+		/* (see comment below) */
 		if (ctx->fi->state != NACC_ST_TX_CELL_CHG_CONTINUE)
 			nacc_fsm_state_chg(ctx->fi, NACC_ST_TX_CELL_CHG_CONTINUE);
 		return;
+	} else if (!neigh_key_present) {
+		/* In case no neighbour key information is present, (This would be the case for UTRAN or EUTRAN cells)
+		 * then we will not provide any system information. Instead we will send the PacketCellChangeContinue
+		 * message immediately. This also applies in the case of re-transmissions. See also: 3GPP TS 48.018,
+		 * section 8c.6.1. */
+		nacc_fsm_state_chg(ctx->fi, NACC_ST_TX_CELL_CHG_CONTINUE);
+		return;
 	}
+
 	/* If tgt cell changed, restart resolving it */
 	if (!neigh_cache_entry_key_eq(&ctx->neigh_key, &neigh_key)) {
 		ctx->neigh_key = neigh_key;
+		ctx->neigh_key_present = neigh_key_present;
 		nacc_fsm_state_chg(ctx->fi, NACC_ST_WAIT_RESOLVE_RAC_CI);
 	}
 	/* else: ignore it, it's a dup, carry on what we were doing */
@@ -341,80 +427,32 @@ static void st_initial(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 	struct nacc_fsm_ctx *ctx = (struct nacc_fsm_ctx *)fi->priv;
 	struct gprs_rlcmac_bts *bts = ctx->ms->bts;
 	Packet_Cell_Change_Notification_t *notif;
+	int rc;
 
 	switch (event) {
 	case NACC_EV_RX_CELL_CHG_NOTIFICATION:
 		notif = (Packet_Cell_Change_Notification_t *)data;
-		if (fill_neigh_key_from_bts_pkt_cell_chg_not(&ctx->neigh_key, bts, notif) < 0) {
-			LOGPFSML(fi, LOGL_NOTICE, "TargetCell type=0x%x not supported\n",
-				 notif->Target_Cell.UnionType);
+		rc = fill_neigh_key_from_bts_pkt_cell_chg_not(&ctx->neigh_key, &ctx->neigh_key_present, bts, notif, ctx);
+		if (rc < 0)
 			osmo_fsm_inst_term(fi, OSMO_FSM_TERM_ERROR, NULL);
-		} else {
+		else if (!ctx->neigh_key_present) {
+			/* In case no neighbour key information is present, (This would be the case for UTRAN or EUTRAN
+			 * cells) then we will not provide any system information. Instead we will send the
+			 * PacketCellChangeContinue message immediately. See also: 3GPP TS 48.018, section 8c.6.1. */
+			nacc_fsm_state_chg(fi, NACC_ST_TX_CELL_CHG_CONTINUE);
+		} else
 			nacc_fsm_state_chg(fi, NACC_ST_WAIT_RESOLVE_RAC_CI);
-		}
 		break;
 	default:
 		OSMO_ASSERT(0);
 	}
 }
 
-static int send_neigh_addr_req_ctrl_iface(struct nacc_fsm_ctx *ctx)
-{
-	struct gprs_rlcmac_bts *bts = ctx->ms->bts;
-	struct gprs_pcu *pcu = bts->pcu;
-	struct ctrl_cmd *cmd = NULL;
-	int rc;
-
-	/* We may have changed to this state previously (eg: we are handling
-	 * another Pkt cell Change Notify with different target). Avoid
-	 * re-creating the socket in that case. */
-	if (ctx->neigh_ctrl_conn->write_queue.bfd.fd == -1) {
-		rc = osmo_sock_init2_ofd(&ctx->neigh_ctrl_conn->write_queue.bfd,
-					 AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP,
-					 NULL, 0, pcu->vty.neigh_ctrl_addr, pcu->vty.neigh_ctrl_port,
-					 OSMO_SOCK_F_CONNECT);
-		if (rc < 0) {
-			LOGPFSML(ctx->fi, LOGL_ERROR,
-				"Failed to establish CTRL (neighbor resolution) connection to BSC r=%s:%u\n\n",
-				pcu->vty.neigh_ctrl_addr, pcu->vty.neigh_ctrl_port);
-			goto err_term;
-		}
-	}
-
-	cmd = ctrl_cmd_create(ctx, CTRL_TYPE_GET);
-	if (!cmd) {
-		LOGPFSML(ctx->fi, LOGL_ERROR, "CTRL msg creation failed\n");
-		goto err_term;
-	}
-
-	cmd->id = talloc_asprintf(cmd, "%u", arfcn_bsic_2_ctrl_id(ctx->neigh_key.tgt_arfcn,
-								  ctx->neigh_key.tgt_bsic));
-	cmd->variable = talloc_asprintf(cmd, "neighbor_resolve_cgi_ps_from_lac_ci.%d.%d.%d.%d",
-					ctx->neigh_key.local_lac, ctx->neigh_key.local_ci,
-					ctx->neigh_key.tgt_arfcn, ctx->neigh_key.tgt_bsic);
-	rc = ctrl_cmd_send(&ctx->neigh_ctrl_conn->write_queue, cmd);
-	if (rc) {
-		LOGPFSML(ctx->fi, LOGL_ERROR, "CTRL msg sent failed: %d\n", rc);
-		goto err_term;
-	}
-
-	talloc_free(cmd);
-	return 0;
-
-err_term:
-	talloc_free(cmd);
-	return -1;
-}
-
 static int send_neigh_addr_req(struct nacc_fsm_ctx *ctx)
 {
 	struct gprs_rlcmac_bts *bts = ctx->ms->bts;
 
-	/* If PCU was configured to use the old CTRL interface, use it: */
-	if (ctx->neigh_ctrl_conn)
-		return send_neigh_addr_req_ctrl_iface(ctx);
-
-	/* Otherwise, by default the new PCUIF over IPA Abis multiplex proto should be used: */
+	/* Using PCUIF over IPA Abis multiplex proto: */
 	return pcu_tx_neigh_addr_res_req(bts, &ctx->neigh_key);
 }
 
@@ -434,7 +472,6 @@ static void st_wait_resolve_rac_ci_on_enter(struct osmo_fsm_inst *fi, uint32_t p
 	}
 
 	/* CGI-PS not in cache, resolve it using BSC Neighbor Resolution CTRL interface */
-
 	LOGPFSML(fi, LOGL_DEBUG, "No CGI-PS found in cache, resolving " NEIGH_CACHE_ENTRY_KEY_FMT "...\n",
 		 NEIGH_CACHE_ENTRY_KEY_ARGS(&ctx->neigh_key));
 
@@ -674,7 +711,8 @@ static struct osmo_fsm_state nacc_fsm_states[] = {
 		.in_event_mask =
 			X(NACC_EV_RX_CELL_CHG_NOTIFICATION),
 		.out_state_mask =
-			X(NACC_ST_WAIT_RESOLVE_RAC_CI),
+			X(NACC_ST_WAIT_RESOLVE_RAC_CI) |
+			X(NACC_ST_TX_CELL_CHG_CONTINUE),
 		.name = "INITIAL",
 		.action = st_initial,
 	},
@@ -833,15 +871,6 @@ static int nacc_fsm_ctx_talloc_destructor(struct nacc_fsm_ctx *ctx)
 		ctx->fi = NULL;
 	}
 
-	if (ctx->neigh_ctrl_conn) {
-		if (ctx->neigh_ctrl_conn->write_queue.bfd.fd != -1) {
-			osmo_wqueue_clear(&ctx->neigh_ctrl_conn->write_queue);
-			osmo_fd_unregister(&ctx->neigh_ctrl_conn->write_queue.bfd);
-			close(ctx->neigh_ctrl_conn->write_queue.bfd.fd);
-			ctx->neigh_ctrl_conn->write_queue.bfd.fd = -1;
-		}
-	}
-
 	return 0;
 }
 
@@ -859,19 +888,6 @@ struct nacc_fsm_ctx *nacc_fsm_alloc(struct GprsMs* ms)
 	if (!ctx->fi)
 		goto free_ret;
 
-	/* If CTRL ip present, use the old CTRL interface for neighbor resolution */
-	if (ms->bts->pcu->vty.neigh_ctrl_addr) {
-		ctx->neigh_ctrl = ctrl_handle_alloc(ctx, ctx, NULL);
-		ctx->neigh_ctrl->reply_cb = nacc_fsm_ctrl_reply_cb;
-		ctx->neigh_ctrl_conn = osmo_ctrl_conn_alloc(ctx, ctx->neigh_ctrl);
-		if (!ctx->neigh_ctrl_conn)
-			goto free_ret;
-		/* Older versions of osmo_ctrl_conn_alloc didn't properly initialize fd to -1,
-		 * so make sure to do it here otherwise fd may be valid fd 0 and cause trouble */
-		ctx->neigh_ctrl_conn->write_queue.bfd.fd = -1;
-		llist_add(&ctx->neigh_ctrl_conn->list_entry, &ctx->neigh_ctrl->ccon_list);
-	}
-
 	return ctx;
 free_ret:
 	talloc_free(ctx);
